/**
 * @description 是否被定义
 * @param {any} val
 * @returns {Boolean}
 */

export function isDefine(val: any) {
  return val !== void 0 && val !== null
}

/**
 * @description 类型判断
 * @param {any} data
 * @returns {String}
 */
export function typeOf(data: any) {
  return Object.prototype.toString
    .call(data)
    .match(/\[object\s+(.*)\]/)?.[1]
    ?.toLocaleLowerCase()
}

/**
 * @description 对象空属性(undefined | null)过滤
 * @param  {Object} data 过滤对象
 * @returns {Object}
 */
export function filterEmpty(data: Record<string, any>) {
  if (typeOf(data) === 'object') {
    data = Object.entries(data).reduce<Record<string, any>>((d, [k, v]) => {
      if (isDefine(v)) {
        d[k] = v
      }
      return d
    }, {})
  }
  return data
}

/**
 * @description 从content-disposition中提取文件名
 */
export const extractFileName = (str: string) => {
  if (/fileName=([^;\s]*)/gi.test(str)) {
    return decodeURIComponent(RegExp.$1)
  }
  return ''
}

/**
 * @description 文件下载 Blob
 */
export const downloadBlob = (blob: Blob, filename = '') => {
  const a = document.createElement('a')
  a.download = filename
  const blobUrl = URL.createObjectURL(blob)
  a.href = blobUrl
  document.body.appendChild(a)
  a.click()
  a.remove()
  URL.revokeObjectURL(blobUrl)
}

/**
 *
 * @param { String } eleName // elementName
 * @returns { HTMLElement }
 */
export const createFakeDom = <K extends keyof HTMLElementTagNameMap>(eleName: K) => {
  const dom = document.createElement(eleName)
  dom.style.position = 'fixed'
  dom.style.zIndex = '-999999'
  return dom
}
